/* This file is part of the FaCT++ DL reasoner
Copyright (C) 2010-2015 by Dmitry Tsarkov

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "FppTestBase.h"

typedef std::vector<FppTestBase*> TestVec;
typedef TestVec::iterator iterator;

TestVec Tests;

// init c'tor
FppTestBase :: FppTestBase ( const std::string& n )
	: name(n)
{
	fpp.clearKB();
	// define top/bottom role names
	fpp.setTopBottomRoleNames ( "*UROLE*", "*EROLE*", "*UDROLE*", "*EDROLE*" );
	// locally save expression manager
	pEM = fpp.getExpressionManager();
	// do not allow undefined names
	fpp.setUseUndefinedNames(false);

	// save newly created object
	Tests.push_back(this);
}

unsigned int nPassed = 0, nFailed = 0;

static bool runTest ( FppTestBase* test )
{
	++nFailed;
	try {
		if ( !test->test() )
		{
			++nPassed;
			--nFailed;
			return false;
		}
		return true;
	}
	catch ( EFPPAssertion ass )
	{
		std::cerr << ass.what() << ". Query is NOT processed\n";
	}
	catch ( EFPPInconsistentKB )
	{
		std::cerr << "Ontology inconsistent. Query is NOT processed\n";
	}
	catch ( EFPPCantRegName crn )
	{
		std::cerr << "Entity name " << crn.getName() << " is undefined in KB\n";
	}
	catch ( EFPPNonSimpleRole nsr )
	{
		std::cerr << "WARNING: KB is incorrect: "
		<< nsr.what() << ". Query is NOT processed\n";
	}
	catch ( EFPPCycleInRIA cir )
	{
		std::cerr << "WARNING: KB is incorrect: "
		<< cir.what() << ". Query is NOT processed\n";
	}
	return true;
}

int main ( void )
{
	for ( iterator p = Tests.begin(), p_end = Tests.end(); p != p_end; ++p )
	{
		FppTestBase* test = *p;
		const char* result = runTest(test) ? "FAILED" : "PASSED";
		std::cout << "Test " << test->getName() << ": " << result << "\n";
	}
	std::cout << "\nTotal: " << nPassed << " pass, " << nFailed << " fail\n";
	return 0;
}
