/* This file is part of the FaCT++ DL reasoner
Copyright (C) 2015 by Dmitry Tsarkov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "FppTestBase.h"

class TestNames: public FppTestBase
{
public:		// interface
		/// init c'tor
	TestNames ( void ) : FppTestBase("names") {}
		/// empty d'tor
	virtual ~TestNames ( void ) {}

		/// test names creation and querying; @return true if failed
	virtual bool test ( void )
	{
		std::cout << "Test for " << getName() << " started\n";
		// create a concept C1
		TConceptExpr* C1 = pEM->Concept("C1");
		// create concepts C2-C4
		C(C2);
		C(C3);
		C(C4);

		// create an ordinary (object) role R1
		TORoleExpr* R1 = pEM->ObjectRole("R1");
		// create an ordinary (object) role R2
		R(R2);

		// create a data role D1
		TDRoleExpr* D1 = pEM->DataRole("D1");
		// create a data role D2
		DR(D2);

		// create an individual I1
		TIndividualExpr* I1 = pEM->Individual("I1");
		// create an individual I2
		I(I2);

		// register that entities in the KB
		// note that C2-C4, R2 and D2 are missing here
		fpp.declare(C1);
		fpp.declare(R1);
		fpp.declare(D1);
		fpp.declare(I1);
		fpp.declare(I2);
		// add axiom C1 [= E R2.C2
		TConceptExpr* Some = pEM->Exists(R2,C2);
		fpp.impliesConcepts(C1,Some);
		// now R2 and C2 are registered

		// add axiom D2 [= D1. D2 became defined
		fpp.impliesDRoles(D2,D1);

		// add axiom C2 = C3 = C4. All of them became defined
		pEM->newArgList();
		pEM->addArg(C2);
		pEM->addArg(C3);
		pEM->addArg(C4);
		fpp.equalConcepts();

		// now realise KB (there are individuals so "classify" is not enough)
		fpp.realiseKB();

		//
		// Queries to the hierarchy
		//

		// get all concept names in the KB

		actor.clear();
		// looking for concepts
		actor.needConcepts();
		// get the indirect sub-concepts of TOP; that would be all concepts
		fpp.getSubConcepts ( pEM->Top(), /*direct=*/false, actor );
		// count them; should be 5 (C1-C4 and BOTTOM)
		printActor1D(std::cout);
		if ( array1D.size() != 5 )
			return true;

		// get the direct sub-concepts of TOP
		actor.clear();
		actor.needConcepts();
		fpp.getSupConcepts ( pEM->Bottom(), /*direct=*/true, actor );
		// count them; now should be 4, BOTTOM is not a direct sub-concept
		printActor1D(std::cout);
		if ( array1D.size() != 4 )
			return true;
		// print them in groups
		printActor2D(std::cout);
		// count the groups; should be 2: C1 and C2=C3=C4
		if ( array2D.size() != 3 )
			return true;






//		std::ofstream Out("dl.res");
//		fpp.writeReasoningResult ( Out, 0 );
		return false;
	}
}; // TestNames

// create an instance of the test
TestNames test;

